unit ufrmApp;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Spin;

type
  TfrmApp = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    Label1: TLabel;
    Edit1: TEdit;
    cmdColour: TButton;
    ColorDialog1: TColorDialog;
    Label2: TLabel;
    SpinEdit1: TSpinEdit;
    Label3: TLabel;
    ComboBox1: TComboBox;
    Label4: TLabel;
    ComboBox2: TComboBox;
    Label5: TLabel;
    Edit2: TEdit;
    cmdBrushColour: TButton;
    Label6: TLabel;
    ComboBox3: TComboBox;
    Label7: TLabel;
    Edit3: TEdit;
    cmdBrushBitmap: TButton;
    OpenDialog1: TOpenDialog;
    FontDialog1: TFontDialog;
    cmdFont: TButton;
    Edit4: TEdit;
    Button1: TButton;
    Label8: TLabel;
    procedure cmdColourClick(Sender: TObject);
    procedure SpinEdit1Changed(Sender: TObject; NewValue: Integer);
    procedure ComboBox1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ComboBox2Change(Sender: TObject);
    procedure cmdBrushColourClick(Sender: TObject);
    procedure ComboBox3Change(Sender: TObject);
    procedure cmdBrushBitmapClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Edit3Change(Sender: TObject);
    procedure cmdFontClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FTargetCanvas: TCanvas;
    FBrushBitmap: TBitmap;
    procedure SetTargetCanvas(const Value: TCanvas);
    { Private declarations }
  public
    { Public declarations }
    property TargetCanvas: TCanvas read FTargetCanvas write SetTargetCanvas;
  end;

var
  frmApp: TfrmApp;

implementation

uses
  TypInfo;

{$R *.dfm}

{ TfrmApp }

procedure TfrmApp.SetTargetCanvas(const Value: TCanvas);
begin
  FTargetCanvas := Value;
  Edit1.Color := Value.Pen.Color;
  SpinEdit1.Value := Value.Pen.Width;
  ComboBox1.ItemIndex := Ord(Value.Pen.Style);
  ComboBox2.ItemIndex := Ord(Value.Pen.Mode);
  Edit2.Color := Value.Brush.Color;
  ComboBox3.ItemIndex := Ord(Value.Brush.Style);
  Edit3.Text := '';     // File name not stored
  Edit4.Color := Value.Font.Color;
end;

procedure TfrmApp.cmdColourClick(Sender: TObject);
begin
  ColorDialog1.Color := TargetCanvas.Pen.Color;
  if ColorDialog1.Execute then
  begin
    TargetCanvas.Pen.Color := ColorDialog1.Color;
    Edit1.Color := ColorDialog1.Color;
  end;
end;

procedure TfrmApp.SpinEdit1Changed(Sender: TObject; NewValue: Integer);
begin
  TargetCanvas.Pen.Width := SpinEdit1.Value;
end;

procedure TfrmApp.ComboBox1Change(Sender: TObject);
begin
  TargetCanvas.Pen.Style :=
    TPenStyle(ComboBox1.ItemIndex);
end;

procedure TfrmApp.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  // Create the TBitmap object
  FBrushBitmap := TBitmap.Create;
  // Load the pen styles into ComboBox1
  for I := Ord(Low(TPenStyle)) to
                    Ord(High(TPenStyle)) do
  begin
    ComboBox1.Items.Add(GetEnumName(
                          TypeInfo(TPenStyle), I));
  end;
  // Load the pen modes into ComboBox2
  for I := Ord(Low(TPenMode)) to
                    Ord(High(TPenMode)) do
  begin
    ComboBox2.Items.Add(GetEnumName(
                          TypeInfo(TPenMode), I));
  end;
  // Load the brush styles into ComboBox3
  for I := Ord(Low(TBrushStyle)) to
                    Ord(High(TBrushStyle)) do
  begin
    ComboBox3.Items.Add(GetEnumName(
                          TypeInfo(TBrushStyle), I));
  end;
end;

procedure TfrmApp.ComboBox2Change(Sender: TObject);
begin
  TargetCanvas.Pen.Mode :=
    TPenMode(ComboBox2.ItemIndex);
end;

procedure TfrmApp.cmdBrushColourClick(Sender: TObject);
begin
  ColorDialog1.Color := TargetCanvas.Brush.Color;
  if ColorDialog1.Execute then
  begin
    TargetCanvas.Brush.Color := ColorDialog1.Color;
    Edit2.Color := ColorDialog1.Color;
  end;
end;

procedure TfrmApp.ComboBox3Change(Sender: TObject);
begin
  TargetCanvas.Brush.Style :=
    TBrushStyle(ComboBox3.ItemIndex);
end;

procedure TfrmApp.cmdBrushBitmapClick(Sender: TObject);
begin
  OpenDialog1.FileName := Edit3.Text;
  if OpenDialog1.Execute then
  begin
    Edit3.Text := OpenDialog1.FileName;
  end;
end;

procedure TfrmApp.FormDestroy(Sender: TObject);
begin
  // Free the bitmap objects
  FBrushBitmap.Free;
end;

procedure TfrmApp.Edit3Change(Sender: TObject);
begin
  if Edit3.Text = '' then
  begin
    TargetCanvas.Brush.Bitmap := nil;
  end
  else
  begin
    if FileExists(Edit3.Text) then
    begin
      FBrushBitmap.LoadFromFile(Edit3.Text);
      TargetCanvas.Brush.Bitmap := FBrushBitmap;
    end;
  end;
end;

procedure TfrmApp.cmdFontClick(Sender: TObject);
begin
  FontDialog1.Font := TargetCanvas.Font;
  if FontDialog1.Execute then
  begin
    TargetCanvas.Font := FontDialog1.Font;
  end;
end;

procedure TfrmApp.Button1Click(Sender: TObject);
begin
  ColorDialog1.Color := TargetCanvas.Brush.Color;
  if ColorDialog1.Execute then
  begin
    TargetCanvas.Font.Color := ColorDialog1.Color;
    Edit4.Color := ColorDialog1.Color;
  end;
end;

end.
