unit ufrmDraw;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls, Spin;

type

  TWiggleState = (dsStraight, dsBent);

  TfrmDraw = class(TForm)
    PageControl1: TPageControl;
    StatusBar1: TStatusBar;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    StatusBar2: TStatusBar;
    Button1: TButton;
    chkRubber: TCheckBox;
    Label1: TLabel;
    edtFileName: TEdit;
    cmdFileName: TButton;
    OpenDialog1: TOpenDialog;
    chkStretch: TCheckBox;
    TabSheet5: TTabSheet;
    Label2: TLabel;
    edtText: TEdit;
    chkTextRect: TCheckBox;
    cmbHAlign: TComboBox;
    cmbVAlign: TComboBox;
    chkImage: TCheckBox;
    TabSheet6: TTabSheet;
    cmdWiggle: TButton;
    Timer1: TTimer;
    spinInterval: TSpinEdit;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmdFileNameClick(Sender: TObject);
    procedure chkImageClick(Sender: TObject);
    procedure cmdWiggleClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure spinIntervalChanged(Sender: TObject; NewValue: Integer);
    procedure spinIntervalChange(Sender: TObject);
  private
    { Private declarations }
    FTargetCanvas: TCanvas;
    FClearRect: TRect;
    FOrigin: TPoint;
    FMoveTo: TPoint;
    FDrawing: boolean;
    FDrawingBitmap: TBitmap;
    FAnimated: boolean;
    FWiggleState: TWiggleState;
    FWiggleOrigin: TPoint;
    procedure DrawSomething(X, Y: Integer);
    procedure DrawPoint(const X, Y: Integer);
    procedure DrawLine(const X, Y: Integer);
    procedure DrawRectangle(const X, Y: Integer);
    procedure DrawBitmap(const X, Y: Integer);
    procedure DrawText(const X, Y: Integer);
    procedure RubberBandLine(const X, Y: Integer;
                const DrawNew: boolean);
    procedure RubberBandRect(const X, Y: Integer;
                const DrawNew: boolean);
    procedure SetTargetCanvas(const Value: TCanvas);
    procedure SetClearRect(const Value: TRect);
    procedure DrawStraight;
    procedure DrawBent;
  public
    { Public declarations }
    procedure MowseDown(Sender: TObject;
                        Button: TMouseButton;
                        Shift: TShiftState;
                        X, Y: Integer);
    procedure MowseMove(Sender: TObject;
                        Shift: TShiftState;
                        X, Y: Integer);
    procedure MowseUp(Sender: TObject;
                      Button: TMouseButton;
                      Shift: TShiftState;
                      X, Y: Integer);
    property TargetCanvas: TCanvas
      read FTargetCanvas
      write SetTargetCanvas;
    property ClearRect: TRect
      read FClearRect
      write SetClearRect;
  end;

var
  frmDraw: TfrmDraw;

implementation

uses
  Math,
  ufrmOutput;

{$R *.dfm}

{ TfrmDraw }

procedure TfrmDraw.SetTargetCanvas(const Value: TCanvas);
begin
  if (FTargetCanvas <> Value) then
  begin
    FTargetCanvas := Value;
  end;
end;

procedure TfrmDraw.FormCreate(Sender: TObject);
begin
  // Create the bitmap object
  FDrawingBitmap := TBitmap.Create;
  FDrawing := False;
  FAnimated := False;
end;

procedure TfrmDraw.DrawSomething(X, Y: Integer);
begin
  case Self.PageControl1.ActivePageIndex of
    0:
    // Draw a point
    DrawPoint(X, Y);
    1:
    // Draw a line
    DrawLine(X, Y);
    2:
    // Draw a rectangle
    DrawRectangle(X, Y);
    3:
    // Draw a bitmap
    DrawBitmap(X, Y);
    4:
    // Draw text
    DrawText(X, Y);
  end;
  StatusBar2.Panels[0].Text :=
    'Drawing at ' +
    IntToStr(TargetCanvas.PenPos.X) + ', ' +
    IntToStr(TargetCanvas.PenPos.Y);

end;

procedure TfrmDraw.DrawPoint(const X, Y: Integer);
begin
  // Draw a point on the canvas
  TargetCanvas.Pixels[X, Y] := TargetCanvas.Pen.Color;
end;

procedure TfrmDraw.DrawLine(const X, Y: Integer);
begin
  // Clear any rubber-band line
  if chkRubber.Checked then
  begin
    // Erase the current rubber-band line
    RubberBandLine(X, Y, False);
    // Move back to the start of the line
    TargetCanvas.MoveTo(FOrigin.X, FOrigin.Y);
  end;
  // Draw a line from the current pen position
  TargetCanvas.LineTo(X, Y);
end;

procedure TfrmDraw.DrawRectangle(const X, Y: Integer);
begin
  // Clear any rubber-band rectangle
  if chkRubber.Checked then
    RubberBandRect(X, Y, False);
  // Draw a rectangle on the canvas.
  TargetCanvas.Rectangle(TargetCanvas.PenPos.X,
                         TargetCanvas.PenPos.Y,
                         X,
                         Y);
end;

procedure TfrmDraw.Button1Click(Sender: TObject);
var
  tmpBrush: TBrush;
begin
  if Assigned(TargetCanvas) then
  begin
    tmpBrush := TBrush.Create;
    try
      // Save the current brush
      tmpBrush.Assign(TargetCanvas.Brush);
      // Set the brush to clear the window
      TargetCanvas.Brush.Color := clWhite;
      TargetCanvas.Brush.Style := bsSolid;
      TargetCanvas.FillRect(ClearRect);
      // Restore the original settings
      TargetCanvas.Brush.Assign(tmpBrush);
    finally
      tmpBrush.Free;
    end;
  end;
end;

procedure TfrmDraw.SetClearRect(const Value: TRect);
begin
  FClearRect := Value;
end;

procedure TfrmDraw.MowseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FOrigin.X := X;
  FOrigin.Y := Y;
  FMoveTo.X := X;
  FMoveTo.Y := Y;
  TargetCanvas.MoveTo(X, Y);
  FDrawing := True;
end;

procedure TfrmDraw.MowseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  // Draw a rubber-banded shape as we drag the mouse
  if FDrawing and chkRubber.Checked then
  begin
    case Self.PageControl1.ActivePageIndex of
      1:
      // Draw a rubber-band line
      RubberBandLine(X, Y, True);
      2:
      // Draw a rubber-band rectangle
      RubberBandRect(X, Y, True);
      3:
      // Draw a rubber-band rectangle
      RubberBandRect(X, Y, True);
      4:
      // Draw a rubber-band rectangle
      RubberBandRect(X, Y, True);
    end;
  end;
end;

procedure TfrmDraw.MowseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  DrawSomething(X, Y);
  FDrawing := False;
end;

procedure TfrmDraw.FormDestroy(Sender: TObject);
begin
  // Free the bitmap object
  FreeAndNil(FDrawingBitmap);
end;

procedure TfrmDraw.cmdFileNameClick(Sender: TObject);
begin
  // Load a bitmap into the bitmap object
  OpenDialog1.FileName := edtFileName.Text;
  if OpenDialog1.Execute then
  begin
    edtFileName.Text := OpenDialog1.FileName;
    FDrawingBitmap.LoadFromFile(edtFileName.Text);
  end;
end;

procedure TfrmDraw.DrawBitmap(const X, Y: Integer);
begin
  if not FDrawingBitmap.Empty then
  begin
    if chkStretch.Checked then
    begin
      TargetCanvas.StretchDraw(
        Rect(TargetCanvas.PenPos.X,
             TargetCanvas.PenPos.Y,
             X, Y
            ),
        FDrawingBitmap);
    end
    else
    begin
      TargetCanvas.Draw(X, Y, FDrawingBitmap);
    end;
  end;
end;

procedure TfrmDraw.RubberBandLine(const X, Y: Integer;
  const DrawNew: boolean);
var
  PenMode: TPenMode;
begin
  // Draw a "rubber-banded" line while we drag
  // Save the current pen mode
  PenMode := TargetCanvas.Pen.Mode;
  try
    // Set the pen mode so we can erase it
    TargetCanvas.Pen.Mode := pmNotXor;
    // Move pen back to start point
    TargetCanvas.MoveTo(FOrigin.X, FOrigin.Y);
    // Erase the old line by drawing over it again
    TargetCanvas.LineTo(FMoveTo.X, FMoveTo.Y);
    if DrawNew then
    begin
      // Go back to start point again
      TargetCanvas.MoveTo(FOrigin.X, FOrigin.Y);
      // Draw a new line
      TargetCanvas.LineTo(X, Y);
    end;
    // Remember where we finished
    FMoveTo := Point(X, Y);
  finally
    // Restore the original pen mode
    TargetCanvas.Pen.Mode := PenMode;
  end;
end;

procedure TfrmDraw.RubberBandRect(const X, Y: Integer;
  const DrawNew: boolean);
var
  PenMode: TPenMode;
  tmpBrush: TBrush;
begin
  // Draw a "rubber-banded" line while we drag
  // Save the current pen and brush modes
  PenMode := TargetCanvas.Pen.Mode;
  tmpBrush := TBrush.Create;
  tmpBrush.Assign(TargetCanvas.Brush);
  try
    // Set the pen mode so we can erase it
    TargetCanvas.Pen.Mode := pmNotXor;
    // Set the brush style to bsClear, so it
    // doesn't fill the rectangle when we draw it
    TargetCanvas.Brush.Style := bsClear;
    // Move pen back to start point
    TargetCanvas.MoveTo(FOrigin.X, FOrigin.Y);
    // Erase the old rectangle by drawing over it again
    TargetCanvas.Rectangle(FOrigin.X, FOrigin.Y,
                           FMoveTo.X, FMoveTo.Y);
    if DrawNew then
    begin
      // Go back to start point again
      TargetCanvas.MoveTo(FOrigin.X, FOrigin.Y);
      // Draw a new rectangle
      TargetCanvas.Rectangle(FOrigin.X, FOrigin.Y,
                             X, Y);
    end;
    // Remember where we finished
    FMoveTo := Point(X, Y);
  finally
    // Restore the original pen and brush modes
    TargetCanvas.Pen.Mode := PenMode;
    TargetCanvas.Brush.Assign(tmpBrush);
    tmpBrush.Free;
  end;
end;

procedure TfrmDraw.DrawText(const X, Y: Integer);
{var
  Align: Integer;}
begin
  // Clear any rubber-band rectangle
  if chkRubber.Checked then
    RubberBandRect(X, Y, False);
  // Draw the text on the canvas
  if chkTextRect.Checked then
  begin
    // Draw inside a clipping rectangle
{    // Set the horizontal alignment bit
    Align := 1 shl cmbHAlign.ItemIndex;
    // Set the vertical alignment bit
    Align := Align +
            (1 shl (cmbVAlign.ItemIndex + 3));}
    TargetCanvas.TextRect(
      Rect(FOrigin.X, FOrigin.Y, X, Y),
      FOrigin.X,
      FOrigin.Y,
      edtText.Text);
//      Align);               // Kylix parameter not available in Delphi
  end
  else
  begin
    // Don't clip
    TargetCanvas.TextOut(X, Y, edtText.Text);
  end;
end;

procedure TfrmDraw.chkImageClick(Sender: TObject);
begin
  frmOutput.Image1.Visible := chkImage.Checked;
end;

procedure TfrmDraw.cmdWiggleClick(Sender: TObject);
begin
  if not FAnimated then
  begin
    FWiggleOrigin := Point(0, ClearRect.Bottom div 2);
    DrawStraight;
    FWiggleState := dsStraight;
    Timer1.Enabled := True;
    FAnimated := True;
    cmdWiggle.Caption := 'Stop';
  end
  else
  begin
    FAnimated := False;
  end;
end;

procedure TfrmDraw.Timer1Timer(Sender: TObject);
begin
  if FAnimated then
  begin
    case FWiggleState of
      dsStraight:
      begin
        // Wiggle is straight. Erase straight &
        // draw bent
        DrawStraight;
        // Move wiggle origin
        FWiggleOrigin.X := FWiggleOrigin.X + 10;
        if FWiggleOrigin.X > ClearRect.Right then
          FWiggleOrigin.X := 0;
        DrawBent;
        // Signal change of state
        FWiggleState := dsBent;
      end;
      dsBent:
      begin
        // Wiggle is bent. Erase bent &
        // draw straight
        DrawBent;
        DrawStraight;
        // Signal change of state
        FWiggleState := dsStraight;
      end;
    end;
  end
  else
  begin
    case FWiggleState of
      dsStraight:
      begin
        // Erase the straight wiggle
        DrawStraight;
      end;
      dsBent:
      begin
        // Erase the bent wiggle
        DrawBent;
      end;
    end;
    // Stop animation
    Timer1.Enabled := False;
    cmdWiggle.Caption := 'Wiggle';
  end;
end;

procedure TfrmDraw.DrawBent;
var
  tmpPen: TPen;
begin
  // Draw a wide, bent line
  // Save the current pen property values
  tmpPen := TPen.Create;
  try
    tmpPen.Assign(TargetCanvas.Pen);
    // Set a wide, black line
    TargetCanvas.Pen.Width := 5;
    TargetCanvas.Pen.Color := clBlack;
    // Set the mode for animation
    TargetCanvas.Pen.Mode := pmNotXor;
    // Draw a bent line
    // First part
    TargetCanvas.MoveTo(FWiggleOrigin.X,
                        FWiggleOrigin.Y);
    TargetCanvas.LineTo(FWiggleOrigin.X + 15,
                        FWiggleOrigin.Y - 10);
    // SecondPart
    TargetCanvas.LineTo(FWiggleOrigin.X + 30,
                        FWiggleOrigin.Y);
    // Restore pen property values
    TargetCanvas.Pen.Assign(tmpPen);
  finally
    tmpPen.Free;
  end;

end;

procedure TfrmDraw.DrawStraight;
var
  tmpPen: TPen;
begin
  // Draw a wide, straight line
  // Save the current pen property values
  tmpPen := TPen.Create;
  try
    tmpPen.Assign(TargetCanvas.Pen);
    // Set a wide, black line
    TargetCanvas.Pen.Width := 5;
    TargetCanvas.Pen.Color := clBlack;
    // Set the mode for animation
    TargetCanvas.Pen.Mode := pmNotXor;
    // Draw a straight line
    TargetCanvas.MoveTo(FWiggleOrigin.X,
                        FWiggleOrigin.Y);
    TargetCanvas.LineTo(FWiggleOrigin.X + 40,
                        FWiggleOrigin.Y);
    // Restore pen property values
    TargetCanvas.Pen.Assign(tmpPen);
  finally
    tmpPen.Free;
  end;
end;

procedure TfrmDraw.spinIntervalChanged(Sender: TObject; NewValue: Integer);
begin
  Timer1.Interval := spinInterval.Value;
end;

procedure TfrmDraw.spinIntervalChange(Sender: TObject);
begin
  try
    Timer1.Interval := StrToInt(SpinInterval.Text);
  except
    on E: EConvertError do ShowMessage(SpinInterval.Text +
                                              ' is not a valid integer');
  end;
end;

end.
